﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Microsoft.WindowsAzure.StorageClient;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.ServiceRuntime;
using System.Threading;

namespace Web.Controllers
{
    public class HomeController : Controller
    {
        private static TableServiceContext context = null;
        private static Counter counter = null;
        private static int count = 0;
        private static Thread updateThread = null;

        public static void Initialize()
        {
            if (context == null)
            {
                var tables = CloudStorageAccount.Parse(RoleEnvironment.GetConfigurationSettingValue("DataConnectionString")).CreateCloudTableClient();
                tables.CreateTableIfNotExist("Counters");
                context = tables.GetDataServiceContext();
            }
            if (counter == null)
            {
                try
                {
                    counter = context.CreateQuery<Counter>("Counters").Where(c => c.PartitionKey == RoleEnvironment.DeploymentId && c.RowKey == RoleEnvironment.CurrentRoleInstance.Id).Single();
                }
                catch
                {
                    counter = new Counter();
                    context.AddObject("Counters", counter);
                    context.SaveChangesWithRetries();
                    count = 0;
                }
            }
            if (updateThread == null)
            {
                updateThread = new Thread(new ThreadStart(() =>
                {
                    while (true)
                    {
                        if (counter.Count != count)
                        {
                            counter.Count = count;
                            context.UpdateObject(counter);
                            context.SaveChangesWithRetries();
                        }
                        Thread.Sleep(TimeSpan.FromSeconds(5));
                    }
                }));
                updateThread.Start();
            }
        }

        public ActionResult Index()
        {
            return View(CurrentCount());
        }

        private int CurrentCount()
        {
            return context.CreateQuery<Counter>("Counters").AsTableServiceQuery().ToList().Sum(c => c.Count);
        }

        [HttpPost]
        public JsonResult GetCount()
        {
            return Json(CurrentCount());
        }

        [HttpPost]
        public ActionResult IncrementCount()
        {
            Interlocked.Increment(ref count);
            return new EmptyResult();
        }
    }
}
